// Modified by Keerthi K Sathi
// 01/27/06 CS 202 Homework #1
// Fig. 7.16: GradeBook.h
// Definition of class GradeBook that uses an array to store test grades.
// Member functions are defined in GradeBook.cpp

#include <string> // program uses C++ Standard Library string class
#include<iostream>
using std::cout;
using std::cin;
using std::string;

// GradeBook class definition
class GradeBook
{
public:
   // constant -- number of students who took the test
   const static int students = 10; // note public data

   // constructor initializes course name and array of grades
   GradeBook( string, const int [] );

   void setCourseName( string ); // function to set the course name
   string getCourseName(); // function to retrieve the course name
   void displayMessage(); // display a welcome message
   void processGrades(int); // perform various operations on the grade data
   int getMinimum(int); // find the minimum grade for the test
   int getMaximum(int); // find the maximum grade for the test
   double getAverage(int); // determine the average grade for the test
   void outputBarChart(int); // output bar chart of grade distribution
   void outputGrades(int); // output the contents of the grades array
private:
   string courseName; // course name for this grade book
   int grades[ students ]; // array of student grades
}; // end class GradeBook  

/**************************************************************************
 * (C) Copyright 1992-2005 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 **************************************************************************/

// Modified by Keerthi K Sathi
// 01/27/06 CS 202 Homework #1
// Fig. 7.17: GradeBook.cpp
// Member-function definitions for class GradeBook that
// uses an array to store test grades.
#include <iostream>
using std::cout;
using std::cin;
using std::endl;
using std::fixed;        

#include <iomanip>
using std::setprecision;
using std::setw;

#include "GradeBook.h" // GradeBook class definition

// constructor initializes courseName and grades array
GradeBook::GradeBook( string name, const int gradesArray[] )
{
   setCourseName( name ); // initialize courseName

   // copy grades from gradeArray to grades data member
   for ( int grade = 0; grade < students; grade++ )
      grades[ grade ] = gradesArray[ grade ];
} // end GradeBook constructor

// function to set the course name
void GradeBook::setCourseName( string name )
{
   courseName = name; // store the course name
} // end function setCourseName

// function to retrieve the course name
string GradeBook::getCourseName()
{
   return courseName;
} // end function getCourseName

// display a welcome message to the GradeBook user
void GradeBook::displayMessage()
{
   // this statement calls getCourseName to get the 
   // name of the course this GradeBook represents
   cout << "Welcome to the grade book for\n" << getCourseName() << "!" 
      << endl;
} // end function displayMessage

// perform various operations on the data
void GradeBook::processGrades(int count)
{
   // output grades array
   outputGrades(count);

   // call function getAverage to calculate the average grade
   cout << "\nClass average is " << setprecision( 2 ) << fixed <<
      getAverage(count) << endl;

   // call functions getMinimum and getMaximum
   cout << "Lowest grade is " << getMinimum(count) << "\nHighest grade is "
      << getMaximum(count) << endl;

   // call function outputBarChart to print grade distribution chart
   outputBarChart(count);
} // end function processGrades

// find minimum grade
int GradeBook::getMinimum(int count)
{
   int lowGrade = 100; // assume lowest grade is 100

   // loop through grades array
   for ( int grade = 0; grade < count; grade++ )
   {
      // if current grade lower than lowGrade, assign it to lowGrade
      if ( grades[ grade ] < lowGrade )
         lowGrade = grades[ grade ]; // new lowest grade
   } // end for

   return lowGrade; // return lowest grade
} // end function getMinimum

// find maximum grade
int GradeBook::getMaximum(int count)
{
   int highGrade = 0; // assume highest grade is 0

   // loop through grades array
   for ( int grade = 0; grade < count; grade++ )
   {
      // if current grade higher than highGrade, assign it to highGrade
      if ( grades[ grade ] > highGrade )
         highGrade = grades[ grade ]; // new highest grade
   } // end for

   return highGrade; // return highest grade
} // end function getMaximum     

// determine average grade for test
double GradeBook::getAverage(int count)
{
   int total = 0; // initialize total

   // sum grades in array
   for ( int grade = 0; grade < count; grade++ )
      total += grades[ grade ];

   // return average of grades
   return static_cast< double >( total ) / count;
} // end function getAverage

// output bar chart displaying grade distribution
void GradeBook::outputBarChart(int count)
{
   cout << "\nGrade distribution:" << endl;

   // stores frequency of grades in each range of 10 grades
   const int frequencySize = 11;
   int frequency[ frequencySize ] = { 0 };

   // for each grade, increment the appropriate frequency
   for ( int grade = 0; grade < count; grade++ )
      frequency[ grades[ grade ] / 10 ]++;

   // for each grade frequency, print bar in chart
   for ( int count = 0; count < frequencySize; count++ ) 
   {
      // output bar labels ("0-9:", ..., "90-99:", "100:" )
      if ( count == 0 )
         cout << "  0-9: ";
      else if ( count == 10 )
         cout << "  100: ";
      else
         cout << count * 10 << "-" << ( count * 10 ) + 9 << ": ";
      
      // print bar of asterisks
      for ( int stars = 0; stars < frequency[ count ]; stars++ ) 
         cout << '*';

      cout << endl; // start a new line of output
   } // end outer for
} // end function outputBarChart

// output the contents of the grades array
void GradeBook::outputGrades(int count)
{
   cout << "\nThe grades are:\n\n";

   // output each student's grade
   for ( int student = 0; student < count; student++ )
      cout << "Student " << setw( 2 ) << student + 1 << ": " << setw( 3 )
         << grades[ student ] << endl;
} // end function outputGrades


/**************************************************************************
 * (C) Copyright 1992-2005 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 **************************************************************************/

// Modified by Keerthi Kanth Reddy Sathi
// 01/27/06
// CS202 Homework #1
// Fig. 7.18: fig07_18.cpp
// Creates GradeBook object using an array of grades.

#include "GradeBook.h" // GradeBook class definition

// function main begins program execution
int main()
{
	int gradesArray[ GradeBook::students ];
	int k=0,count=0;

	cout << "Enter the grades; end your input with -1 if the total no of grades are less than 10 : ";

	for (int i=0;i<10;i++)
	{
		cin >> k; //Store the entered value in a temperory variable
		if ( k == -1) // check if the input has ended
			break;
		else
		{
		gradesArray[i] = k; //store the entered value in the array
		count++; //increment the count each time a value is stored in the array
		}
	}

   GradeBook myGradeBook( 
      "CS101 Introduction to C++ Programming", gradesArray );
   myGradeBook.displayMessage();
   myGradeBook.processGrades(count);  
   return 0;
} // end main


/**************************************************************************
 * (C) Copyright 1992-2005 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 **************************************************************************/

/* OUTPUT:

Enter the grades, end your input with -1 if the total no of grades are less than 10 :
10 90 20 80 30 70 40 60 50 -1
Welcome to the grade book for
CS101 Introduction to C++ Programming!

The grades are:

Student  1:  10
Student  2:  90
Student  3:  20
Student  4:  80
Student  5:  30
Student  6:  70
Student  7:  40
Student  8:  60
Student  9:  50

Class average is 50.00
Lowest grade is 10
Highest grade is 90

Grade distribution:
  0-9:
10-19: *
20-29: *
30-39: *
40-49: *
50-59: *
60-69: *
70-79: *
80-89: *
90-99: *
  100:
Press any key to continue

*/

